@@ -1,11 +1,12 @@
 #![cfg(feature = "test-support")]
 use std::env;
 use std::process::Command;
-use std::sync::atomic::{AtomicBool, Ordering};
+use std::sync::atomic::{AtomicBool, AtomicPtr, Ordering};
 
 use crate::core::mark_initialized;
 
 static TEST_MODE: AtomicBool = AtomicBool::new(false);
+static TEST_MODULE: AtomicPtr<String> = AtomicPtr::new(std::ptr::null_mut());
 
 // we need this.
 pub use ctor::ctor;
@@ -27,7 +28,9 @@ macro_rules! enable_test_support {
         #[$crate::testsupport::ctor]
         #[used]
         fn __procspawn_test_support_init() {
-            $crate::testsupport::enable();
+            // strip the crate name from the module path
+            let module_path = std::module_path!().splitn(2, "::").nth(1);
+            $crate::testsupport::enable(module_path);
         }
 
         #[test]
@@ -37,8 +40,16 @@ macro_rules! enable_test_support {
     };
 }
 
-pub fn enable() {
-    TEST_MODE.store(true, Ordering::SeqCst);
+pub fn enable(module: Option<&str>) {
+    if TEST_MODE.swap(true, Ordering::SeqCst) {
+        panic!("procspawn testmode can only be enabled once");
+    }
+
+    if let Some(module) = module {
+        let ptr = Box::into_raw(Box::new(module.to_string()));
+        TEST_MODULE.store(ptr, Ordering::SeqCst);
+    }
+
     mark_initialized();
 }
 
@@ -47,9 +58,16 @@ pub struct TestMode {
     pub should_silence_stdout: bool,
 }
 
+fn test_helper_path() -> String {
+    match unsafe { TEST_MODULE.load(Ordering::SeqCst).as_ref() } {
+        Some(module) => format!("{}::procspawn_test_helper", module),
+        None => "procspawn_test_helper".to_string(),
+    }
+}
+
 pub fn update_command_for_tests(cmd: &mut Command) -> Option<TestMode> {
     if TEST_MODE.load(Ordering::SeqCst) {
-        cmd.arg("procspawn_test_helper");
+        cmd.arg(test_helper_path());
         cmd.arg("--exact");
         cmd.arg("--test-threads=1");
         cmd.arg("-q");
